import os
import requests

import cv2


DIRECTORY = "lab-4"

# Инициализация детектора лиц YuNET
if not os.path.exists(f"{DIRECTORY}/face_detection_yunet_2023mar.onnx"):
    print("Загрузка face_detection_yunet_2023mar.onnx ...")
    url = "https://github.com/opencv/opencv_zoo/raw/refs/heads/main/models/face_detection_yunet/face_detection_yunet_2023mar.onnx"
    response = requests.get(url)
    with open(f"{DIRECTORY}/face_detection_yunet_2023mar.onnx", "wb") as f:
        f.write(response.content)
    print("Загрузка face_detection_yunet_2023mar.onnx завершена")

detector = cv2.FaceDetectorYN.create(
    f"{DIRECTORY}/face_detection_yunet_2023mar.onnx", "", (320, 320), 0.9, 0.3, 5000
)

# Инициализация распознавателя лиц
if not os.path.exists(f"{DIRECTORY}/face_recognition_sface_2021dec.onnx"):
    print("Загрузка face_recognition_sface_2021dec.onnx ...")
    url = "https://github.com/opencv/opencv_zoo/raw/refs/heads/main/models/face_recognition_sface/face_recognition_sface_2021dec.onnx"
    response = requests.get(url)
    with open(f"{DIRECTORY}/face_recognition_sface_2021dec.onnx", "wb") as f:
        f.write(response.content)
    print("Загрузка face_recognition_sface_2021dec.onnx завершена")

recognizer = cv2.FaceRecognizerSF.create(
    f"{DIRECTORY}/face_recognition_sface_2021dec.onnx", ""
)

# Загрузка референсного изображения
ref_image = cv2.imread(f"{DIRECTORY}/task_1_reference.jpg")
if ref_image is None:
    print("Не удалось загрузить референсное изображение")
    exit()

# Обнаружение лица на референсном изображении
height, width = ref_image.shape[:2]
detector.setInputSize((width, height))
_, faces = detector.detect(ref_image)

if faces is None:
    print("На референсном изображении не найдено лиц")
    exit()

# Извлечение признаков референсного лица
ref_face = faces[0]
aligned_ref_face = recognizer.alignCrop(ref_image, ref_face)
ref_feature = recognizer.feature(aligned_ref_face)

# Инициализация видеопотока с веб-камеры
cap = cv2.VideoCapture(0)
if not cap.isOpened():
    print("Не удалось открыть веб-камеру")
    exit()

# Порог косинусного сходства для распознавания
COSINE_THRESHOLD = 0.4
L2_THRESHOLD = 1.2

while True:
    ret, frame = cap.read()
    if not ret:
        print("Не удалось получить кадр")
        break

    # Обнаружение лиц в кадре
    frame_height, frame_width = frame.shape[:2]
    detector.setInputSize((frame_width, frame_height))
    _, faces = detector.detect(frame)

    if faces is not None:
        for face in faces:
            # Получение координат ограничивающего прямоугольника
            bbox = list(map(int, face[:4]))
            x, y, w, h = bbox

            # Извлечение признаков текущего лица
            aligned_face = recognizer.alignCrop(frame, face)
            current_feature = recognizer.feature(aligned_face)

            # Сравнение с референсным лицом
            cosine_score = recognizer.match(
                ref_feature, current_feature, cv2.FaceRecognizerSF_FR_COSINE
            )
            l2_score = recognizer.match(
                ref_feature, current_feature, cv2.FaceRecognizerSF_FR_NORM_L2
            )

            # Определение цвета прямоугольника
            color = (0, 0, 255)  # Красный по умолчанию
            if cosine_score >= COSINE_THRESHOLD or l2_score <= L2_THRESHOLD:
                print(f"{cosine_score=:.3f}, {l2_score=:.3f}")
                if cosine_score >= COSINE_THRESHOLD and l2_score <= L2_THRESHOLD:
                    print("Сработали оба порога")
                elif cosine_score >= COSINE_THRESHOLD:
                    print("Сработал порог косинусного сходства")
                elif l2_score <= L2_THRESHOLD:
                    print("Сработал порог L2 сходства")
                color = (0, 255, 0)  # Зеленый для референсного лица

            # Отрисовка прямоугольника
            cv2.rectangle(frame, (x, y), (x + w, y + h), color, 2)

    # Отображение результата
    cv2.imshow("Face Detection", frame)

    # Выход по нажатию 'q'
    if cv2.waitKey(1) & 0xFF == ord("q"):
        break

# Освобождение ресурсов
cap.release()
cv2.destroyAllWindows()
